###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.11 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import rfc822_attachment
import rfc822_address_container
import rfc822_address
import base_item
import cStringIO
import mimetools
import MimeWriter
import string

add_rfc822_message_redirect = HTMLFile('rfc822_message_add_redirect', globals())
manage_add_rfc822_message_form = HTMLFile('rfc822_message_add', globals())


def manage_add_rfc822_message(self, id, sender='', reply_to='', body='', subject='', message_id='', REQUEST=None):
    "Add an instance of the rfc822_message class"

    self._setObject(id, rfc822_message(id, sender, body, subject, message_id=message_id, reply_to=reply_to))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class rfc822_message(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'RFC822Message'
	icon = 'misc_/RFC822Message/Folder_icon.gif'

	_properties = (
		{'id':'sender', 'type':'string'},
		{'id':'reply_to', 'type':'string'},
		{'id':'body', 'type':'text'},
		{'id':'readonly', 'type':'boolean'},
		)

	meta_types = (
		{'name':'RFC822Attachment', 'action':'manage_add_rfc822_attachment_form'},
		{'name':'RFC822Address', 'action':'manage_add_rfc822_address_form'},
		{'name':'RFC822AddressContainer', 'action':'manage_add_rfc822_address_container_form'},
		)

	manage_add_rfc822_attachment_form = rfc822_attachment.manage_add_rfc822_attachment_form
	manage_add_rfc822_attachment = rfc822_attachment.manage_add_rfc822_attachment
	rfc822_attachment_add_redirect = rfc822_attachment.rfc822_attachment_add_redirect

	manage_add_rfc822_address_container_form = rfc822_address_container.manage_add_rfc822_address_container_form
	manage_add_rfc822_address_container = rfc822_address_container.manage_add_rfc822_address_container
	rfc822_address_container_add_redirect = rfc822_address_container.rfc822_address_container_add_redirect

	#manage_add_rfc822_address_form = rfc822_address.manage_add_rfc822_address_form
	#manage_add_rfc822_address = rfc822_address.manage_add_rfc822_address
	#rfc822_address_add_redirect = rfc822_address.rfc822_address_add_redirect	

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add RFC822Attachment',
		 ('manage_add_rfc822_attachment', 'manage_add_rfc822_attachment_form')),
		('Add RFC822Address',
		 ('manage_add_rfc822_address', 'manage_add_rfc822_address_form')),
		('View RFC822Messages',
		 ('',)),
		)

	index_html = HTMLFile('rfc822_message_index', globals())

	view_index_html = HTMLFile('rfc822_message_view_index', globals())
	edit_index_html = HTMLFile('rfc822_message_edit_index', globals())


	def copy(self):
		"""Copies message to temporary folder."""

		temporary = self.get_temporary_folder()

		message_copy = self.getParentNode().manage_copyObjects(self.id)
		temporary.manage_pasteObjects(cb_copy_data=message_copy)

		# We do this so we never get id conflicts.

		new_id = str(self.get_unique_id())

		temporary.manage_renameObject(self.id, new_id)

		message_new = temporary[new_id]

		return message_new


	def delete_address_fields(self):
		"""Removes all recipients."""

		for address_field in ['to', 'cc', 'bcc']:

			for address in self[address_field].objectValues():

				self[address_field].manage_delObject(address.id)


	def forward(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for forwarding."""

		message_new = self.copy()

		message_new.subject = self.render_forward_subject()

		message_new.delete_address_fields()

		RESPONSE.redirect(message_new.absolute_url())



	def reply(self, REQUEST=None, RESPONSE=None, all=None):
		"""Method that prepares a message for replying."""

		new_id = str(self.get_unique_id())

		new_body = '> ' + string.join(string.split(self.body,'\n'), '\n> ')

		if self.headers.has_key('references'):

			references = string.split(self.headers['references'])

		else:

			references = []

		headers = {}

		try:

			references.append('<' + self.headers['message-id'] + '>')

		except KeyError:

			# There was no message id..

			pass

		if references:

			headers['references'] = string.join(references, ' ')

		temporary_folder = self.get_temporary_folder()

		manage_add_rfc822_message(temporary_folder, new_id,
								  sender=self.email_address,
								  body=new_body,
								  subject=self.render_reply_subject())

		temporary_folder[new_id]['to'].manage_add_rfc822_address(
			str(self.get_unique_id()), self.sender, comment='', realname='')

		if all is not None:

			for address_field in self.get_filled_address_fields():
				
				for rfc822_address in address_field.objectValues('RFC822Address'):

					temporary_folder[new_id]['cc'].manage_add_rfc822_address(rfc822_address.address)

		RESPONSE.redirect(temporary_folder[new_id].absolute_url())


	def reply_all(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for replying."""

		self.reply(REQUEST=REQUEST, RESPONSE=RESPONSE, all=1)


	def render_reply_subject(self):
		"""Method for rendering an appropriate subject upon replies."""

		subject = self.subject

		if string.lower(subject)[:3] == 're:':

			return subject

		else:

			return 'Re: ' + subject


	def render_forward_subject(self):
		"""Method for rendering an appropriate subject upon forwards."""

		subject = self.subject

		if string.lower(subject)[:4] == 'fwd:':

			return subject

		else:

			return 'Fwd: ' + subject


	def get_filled_address_fields(self):
		"""Returns address fields that contain one or more addresses."""

		address_fields = []

		for address_field in ['to', 'cc', 'bcc']:

			if len(self[address_field].objectValues('RFC822Address')):

				address_fields.append(address_field)

		return address_fields


	def render_sender(self):
		"""Renders the 'from' address field."""

		return self.sender


	def render_address_field(self, field='to'):
		"""Renders the addresses contained in the given field."""

		return_string = ""
		
		for object in self[field].objectValues():

			return_string = return_string + '%s, ' % object.address

		else:

			return_string = return_string[:-2]

		return return_string


	def update(self, REQUEST=None, RESPONSE=None):
		"""Updates message data."""

		self.body = REQUEST['body']
		self.subject = REQUEST['subject']

		if RESPONSE is not None:

			RESPONSE.redirect(self.absolute_url())


	def send(self, REQUEST=None, RESPONSE=None):
		"""Sends the message."""

		if REQUEST is not None:

			self.update(REQUEST=REQUEST)

		# Do message format magic and such calls here

		if RESPONSE:

			return 'success'


	def add_recipient(address_field='to', address='', realname='', comment=''):
		"""Adds a recipient to the given address field."""

		new_id = str(self.get_unique_id())

		self[address_field].manage_add_rfc822_address(new_id, address, comment=comment, realname=realname)


	#def handle_request(self, REQUEST=None):
	#	"""Method we call when sub-objects get form data."""

	#	self.body = REQUEST['body']
	#	self.subject = REQUEST['subject']
		

	def set_header(self, header_name, header_value):
		"""Sets a mail message header."""

		self.headers[header_name] = header_value


	def add_newsgroup(self, newsgroup):
		"""Adds a newsgroup to the message."""

		try:

			self.newsgroups.index(newsgroup)
			return

		except:

			self.newsgroups.append(newsgroup)


	def raw_message(self):
		"""Renders an RFC822 compliant message."""

		rendered_message = cStringIO.StringIO()

		writer = MimeWriter.MimeWriter(rendered_message)

		writer.addheader('From', self.sender)

		for header_key in self.headers.keys():

			writer.addheader(header_key, self.headers[header_key])

		for address_field in self.get_filled_address_fields():

			writer.addheader(address_field, self.render_address_field(field=address_field))

		if self.newsgroups:

			writer.addheader('newsgroups', string.join(self.newsgroups,', '))
		
		if not self.headers.has_key('message-id'):

			writer.addheader('message-id', 'FIXME')

		if len(self.objectValues('RFC822Attachment')):

			writer.addheader('Mime-Version', '1.0')

		writer.addheader('date', self.ZopeTime().rfc822())

		writer.flushheaders()

		if len(self.objectValues('RFC822Attachment')):

			writer.startmultipartbody('mixed')

			mime_part = writer.nextpart()

			mime_part_writer = mime_part.startbody('text/plain')

			mimetools.encode(cStringIO.StringIO(self.body),
							 mime_part_writer, 'quoted-printable')

			for attachment in self.objectValues('RFC822Attachment'):

				mime_part = writer.nextpart()

				tmp_headers = {'content-transfer-encoding':attachment.encoding or 'base64'}
				tmp_headers.update(attachment.headers)

				mime_part_writer.startbody(
					attachment.content_type(),
					[('filename', attachment.filename)],
					tmp_headers)

				mime_part_writer(attachment.encode())

			else:

				writer.lastpart()

		else:

			body_writer = writer.startbody('text/plain', [], # FIXME -- text/plain ?
										   {'Content-Transfer-Encoding':
											'quoted-printable'})

			body = cStringIO.StringIO(self.body)
			body.seek(0)

			mimetools.encode(body, body_writer, 'quoted-printable')

		return rendered_message.getvalue()


	def __init__(self, id, sender, body, subject, message_id='', reply_to=''):
		"Initialize the class instance"

		self.id = id

		self.sender = sender
		self.reply_to = reply_to
		self.body = body
		self.subject = subject
		self.message_id = message_id

		self.newsgroups = []
		self.headers = {}

		self.manage_add_rfc822_address_container('to', title='to')
		self.manage_add_rfc822_address_container('cc', title='cc')
		self.manage_add_rfc822_address_container('bcc', title='bcc')

		self.index_object()


default__class_init__(rfc822_message)
